@@ -27,7 +27,9 @@ from __future__ import absolute_import, print_function
 
 import apt
 import apt_pkg
-import datetime
+import aptsources.distro
+from datetime import datetime
+import distro_info
 import dbus
 from gettext import gettext as _
 import gettext
@@ -38,7 +40,6 @@ from aptdaemon.errors import NotAuthorizedError, TransactionFailed
 import logging
 import threading
 import sys
-import time
 
 import gi
 gi.require_version("Gdk", "3.0")
@@ -51,20 +52,15 @@ from .DialogMirror import DialogMirror
 from .DialogEdit import DialogEdit
 from .DialogCacheOutdated import DialogCacheOutdated
 from .DialogAddSourcesList import DialogAddSourcesList
-from .UbuntuProPage import UbuntuProPage
+from .DialogLivepatchError import DialogLivepatchError
+from .DialogAuth import DialogAuth
 
 import softwareproperties
+from softwareproperties.GoaAuth import GoaAuth
 import softwareproperties.distro
 from softwareproperties.SoftwareProperties import SoftwareProperties
 import softwareproperties.SoftwareProperties
 
-from softwareproperties.gtk.utils import (
-    get_ua_status,
-    get_ua_service_status,
-    current_distro,
-    is_current_distro_lts,
-)
-
 from UbuntuDrivers import detect
 
 if GLib.pyglib_version < (3, 9, 1):
@@ -89,6 +85,8 @@ RESPONSE_ADD = 2
     STORE_VISIBLE
 ) = list(range(5))
 
+LIVEPATCH_TIMEOUT = 1200
+
 
 def error(parent_window, summary, msg):
     """ show a error dialog """
@@ -205,8 +203,8 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         self.show_distro()
         # Setup and show the Additonal Drivers tab
         self.init_drivers()
-        # Setup and show the Ubuntu Pro tab
-        self.init_ubuntu_pro()
+        # Setup and show the LivePatch tab
+        self.init_livepatch()
 
         # Connect to switch-page before setting initial tab. Otherwise the
         # first switch goes unnoticed.
@@ -227,9 +225,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                                                apt_pkg.config.find("Dir::Etc::sourcelist"))):
             self.open_file(file)
 
-        # Make sure window is as small as possible initially.
-        self.window_main.resize(1, 1)
-
     def on_main_notebook_page_switched(self, notebook, page, page_num):
         # On the additional drivers page, don't show the backend revert button.
         if page == self.vbox_drivers:
@@ -380,57 +375,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                 self.vbox_updates.add(checkbox)
             checkbox.show()
 
-        status = get_ua_status()
-        if not is_current_distro_lts():
-                esm_available = False
-                esm_enabled = False
-        else:
-                (infra_available, infra_status) = get_ua_service_status("esm-infra", status=status)
-                (apps_available, apps_status) = get_ua_service_status("esm-apps", status=status)
-                esm_available = bool(infra_available or apps_available)
-                esm_enabled = "enabled" in (infra_status, apps_status)
-        distro = current_distro()
-        if esm_enabled:
-                eol_text = _("Extended Security Maintenance")
-                # EOL date should probably be UA contract expiry.
-                # This is probably sooner than ESM EOL for the distro and
-                # gives software properties dialogs a chance to interact about
-                # renewals if needed.
-                try:
-                        # Ignore timezone to simplify formatting python < 3.7
-                        # 3.7 has datetime.fromisoformat()
-                        dt, _sep, _tz = status.get("expires", "").partition("+")
-                        eol_date = datetime.datetime.strptime(
-                            dt, "%Y-%m-%dT%H:%M:%S"
-                        ).date()
-                except ValueError:
-                        print("Unable to determine UA contract expiry")
-                        eol_date = distro.eol
-        else:
-                eol_text = _("Basic Security Maintenance")
-                eol_date = distro.eol
-        self.label_esm_status.set_markup(eol_text)
-        esm_url = "https://ubuntu.com/esm"  # Non-EOL LTS generic ESM
-        today = datetime.datetime.now().date()
-        if today >= eol_date:
-                if esm_available:
-                        # EOL LTS uses release-specific ESM ubuntu.com/XX-YY
-                        distro_ver = distro.version.replace(' LTS', '')
-                        esm_url = "https://ubuntu.com/%s" % distro_ver.replace(".", "-")
-                eol_expiry_text = _("Ended %s - extend or upgrade now") % eol_date.strftime("%x")
-        elif today >= eol_date - datetime.timedelta(days=60):
-                eol_expiry_text = _("Ends %s - extend or upgrade soon") % eol_date.strftime("%x")
-        else:
-                eol_expiry_text = _("Active until %s") % eol_date.strftime("%x")
-        self.label_eol.set_label(eol_expiry_text)
-        self.label_esm_subscribe.set_markup(
-                "<a href=\"%s\">%s</a>" % (esm_url, _("Extend…"))
-        )
-        self.label_esm_subscribe.set_visible(
-                esm_available and not esm_enabled
-        )
-        eol_expiry_text = _("Ended %s") % eol_date.strftime("%x")
-
         # setup the server chooser
         cell = Gtk.CellRendererText()
         self.combobox_server.pack_start(cell, True)
@@ -804,13 +748,6 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
 
     def on_isv_source_toggled(self, cell_toggle, path, store):
         """Enable or disable the selected channel"""
-
-        #FIXME Gtk is doing something that takes a lock while processing
-        # the events, the polkit auth dialog can't be displayed in return,
-        # we don't have a proper fix and it might require GTK changes,
-        # meanwhile we want a working interface so workaround with a sleep
-        # https://launchpad.net/bugs/1727908
-        time.sleep(0.3)
         #FIXME cdroms need to disable the comps in the childs and sources
         iter = store.get_iter((int(path),))
         source_entry = store.get_value(iter, STORE_SOURCE)
@@ -1098,7 +1035,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
     def on_delete_event(self, widget, args):
         """Close the window if requested"""
         self.on_close_button(widget)
-        return False
+        return self.quit_when_livepatch_responds
 
     def on_close_button(self, widget):
         """Show a dialog that a reload of the channel information is required
@@ -1108,8 +1045,11 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
             d = DialogCacheOutdated(self.window_main,
                                     self.datadir)
             d.run()
-
-        self.quit()
+        if self.waiting_livepatch_response:
+            self.quit_when_livepatch_responds = True
+            self.hide()
+        else:
+            self.quit()
 
     def on_button_add_cdrom_clicked(self, widget):
         """ when a cdrom is requested for adding """
@@ -1314,7 +1254,7 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
                         lmp = self.apt_cache[linux_meta]
                         if not lmp.is_installed:
                             self.driver_changes.append(lmp)
-        except (AttributeError, KeyError):
+        except KeyError:
             pass
 
         if button.get_active():
@@ -1543,5 +1483,164 @@ class SoftwarePropertiesGtk(SoftwareProperties, SimpleGtkbuilderApp):
         else:
             self.label_driver_action.set_label(_("No proprietary drivers are in use."))
 
-    def init_ubuntu_pro(self):
-        self.ubuntu_pro_page = UbuntuProPage(self)
+    #
+    # Livepatch
+    #
+    def init_livepatch(self):
+        self.goa_auth = GoaAuth()
+        self.waiting_livepatch_response = False
+        self.quit_when_livepatch_responds = False
+
+        if not self.is_livepatch_supported():
+            self.grid_livepatch.set_visible(False)
+            return
+
+        self.checkbutton_livepatch.set_active(self.is_livepatch_enabled())
+        self.on_goa_auth_changed()
+
+        # hacky way to monitor if livepatch is enabled or not
+        file = Gio.File.new_for_path(path=self.LIVEPATCH_RUNNING_FILE)
+        self.lp_monitor = file.monitor_file(Gio.FileMonitorFlags.NONE)
+
+        # connect to signals
+        self.handlers[self.goa_auth] = \
+            self.goa_auth.connect('notify', lambda o, p: self.on_goa_auth_changed())
+        self.handlers[self.checkbutton_livepatch] = \
+            self.checkbutton_livepatch.connect('toggled', self.on_checkbutton_livepatch_toggled)
+        self.handlers[self.button_ubuntuone] =  \
+            self.button_ubuntuone.connect('clicked', self.on_button_ubuntuone_clicked)
+        self.handlers[self.lp_monitor] = \
+            self.lp_monitor.connect('changed', self.on_livepatch_status_changed)
+
+    def has_online_accounts(self):
+        try:
+            d = Gio.DesktopAppInfo.new('gnome-online-accounts-panel.desktop')
+            return d != None
+        except Exception:
+            return False
+
+    def is_livepatch_supported(self):
+        distro = aptsources.distro.get_distro()
+        di = distro_info.UbuntuDistroInfo()
+
+        return self.has_online_accounts() and \
+               di.is_lts(distro.codename) and \
+               distro.codename in di.supported(datetime.now().date())
+
+    def on_goa_auth_changed(self):
+        if self.goa_auth.logged:
+            self.button_ubuntuone.set_label(_('Sign Out'))
+
+            if self.goa_auth.token:
+                self.checkbutton_livepatch.set_sensitive(True)
+                self.label_livepatch_login.set_label(_('Signed in as %s' % self.goa_auth.username))
+            else:
+                self.checkbutton_livepatch.set_sensitive(False)
+                text = _('%s isn\'t authorized to use Livepatch.' % self.goa_auth.username)
+                text = "<span color='red'>" +  text + "</span>"
+                self.label_livepatch_login.set_markup(text)
+        else:
+            if self.is_livepatch_enabled() and not self.waiting_livepatch_response:
+                # Allow the user to disable livepatch even if
+                # the account expired (see LP: #1768797)
+                self.checkbutton_livepatch.set_sensitive(True)
+                self.label_livepatch_login.set_label(_('Livepatch is active.'))
+            else:
+                self.checkbutton_livepatch.set_sensitive(False)
+                self.label_livepatch_login.set_label(_('To use Livepatch you need to sign in.'))
+
+            self.button_ubuntuone.set_label(_('Sign In…'))
+
+    def on_livepatch_status_changed(self, file_monitor, file, other_file, event_type):
+        if not self.waiting_livepatch_response:
+            self.checkbutton_livepatch.set_active(self.is_livepatch_enabled())
+            self.on_goa_auth_changed()
+
+    def on_button_ubuntuone_clicked(self, button):
+        if self.goa_auth.logged:
+            self.do_logout()
+        else:
+            self.do_login()
+
+    def do_login(self):
+        try:
+            # Show login dialog!
+            dialog = DialogAuth(self.window_main, self.datadir)
+            response = dialog.run()
+        except Exception as e:
+            logging.error(e)
+            error(self.window_main,
+                  _("Error enabling Canonical Livepatch"),
+                  _("Please check your Internet connection."))
+        else:
+            if response == Gtk.ResponseType.OK:
+                self.goa_auth.login(dialog.account)
+                if self.goa_auth.logged:
+                    self.checkbutton_livepatch.set_active(True)
+
+    def do_logout(self):
+        self.checkbutton_livepatch.set_active(False)
+        self.goa_auth.logout()
+
+    def on_checkbutton_livepatch_toggled(self, checkbutton):
+        if self.waiting_livepatch_response:
+            return
+
+        self.waiting_livepatch_response = True
+
+        token = ''
+        enabled = False
+        if self.checkbutton_livepatch.get_active():
+            enabled = True
+            token = self.goa_auth.token if self.goa_auth.token else ''
+        self.backend.SetLivepatchEnabled(enabled, token,
+                                         reply_handler=self.livepatch_enabled_reply_handler,
+                                         error_handler=self.livepatch_enabled_error_handler,
+                                         timeout=LIVEPATCH_TIMEOUT)
+
+    def livepatch_enabled_reply_handler(self, is_error, prompt):
+        self.sync_checkbutton_livepatch(is_error, prompt)
+
+    def livepatch_enabled_error_handler(self, e):
+        if e._dbus_error_name == 'com.ubuntu.SoftwareProperties.PermissionDeniedByPolicy':
+            logging.error("Authentication canceled, changes have not been saved")
+            self.sync_checkbutton_livepatch(is_error=True, prompt=None)
+        else:
+            self.sync_checkbutton_livepatch(is_error=True, prompt=str(e))
+
+    def sync_checkbutton_livepatch(self, is_error, prompt):
+        if is_error:
+            self.waiting_livepatch_response = False
+            self.checkbutton_livepatch.handler_block(self.handlers[self.checkbutton_livepatch])
+            self.checkbutton_livepatch.set_active(self.is_livepatch_enabled())
+            self.checkbutton_livepatch.handler_unblock(self.handlers[self.checkbutton_livepatch])
+
+            if prompt:
+                dialog = DialogLivepatchError(self.window_main, self.datadir)
+                response = dialog.run(prompt, show_settings_button=self.quit_when_livepatch_responds)
+                if response == DialogLivepatchError.RESPONSE_SETTINGS:
+                    self.window_main.show()
+                    self.quit_when_livepatch_responds = False
+        else:
+            do_dbus_call = False
+            if self.is_livepatch_enabled() and not self.checkbutton_livepatch.get_active():
+                do_dbus_call = True
+                enabled = False
+                token = ''
+            elif not self.is_livepatch_enabled() and self.checkbutton_livepatch.get_active():
+                do_dbus_call = True
+                enabled = True
+                token = self.goa_auth.token if self.goa_auth.token else ''
+            else:
+                self.waiting_livepatch_response = False
+
+            if do_dbus_call:
+                self.backend.SetLivepatchEnabled(enabled, token,
+                                                 reply_handler=self.livepatch_enabled_reply_handler,
+                                                 error_handler=self.livepatch_enabled_error_handler,
+                                                 timeout=LIVEPATCH_TIMEOUT)
+
+        self.on_goa_auth_changed()
+
+        if self.quit_when_livepatch_responds:
+            self.on_close_button(self.button_close)
